@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6calc.c
- * Version    : $Id: ipv6calc.c,v 1.36 2011/02/27 11:27:14 peter Exp $
+ * Version    : $Id: ipv6calc.c,v 1.48 2011/10/08 12:03:44 peter Exp $
  * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
@@ -13,6 +13,7 @@
 #include <string.h>
 #include <stdlib.h> 
 #include <getopt.h> 
+#include <unistd.h>
 
 #include "ipv6calc.h"
 #include "libipv6calc.h"
@@ -33,29 +34,72 @@
 #include "librfc2874.h"
 #include "librfc3056.h"
 #include "librfc3041.h"
+#include "librfc5569.h"
 #include "libeui64.h"
 #include "config.h"
+#include "libieee.h"
+#include "../databases/ieee-oui/dbieee_oui.h"
+#include "../databases/ieee-iab/dbieee_iab.h"
+#include "libipv4addr.h"
+#include "../databases/ipv4-assignment/dbipv4addr_assignment.h"
+#include "libipv6addr.h"
+#include "../databases/ipv6-assignment/dbipv6addr_assignment.h"
+
+#ifdef SUPPORT_IP2LOCATION
+#include "IP2Location.h"
+
+/* 
+ *  * API_VERSION is defined as a bareword in IP2Location.h, 
+ *   * we need this trick to stringify it. Blah.
+ *    */
+#define makestr(x) #x
+#define xmakestr(x) makestr(x)
+#endif
+
+
+#ifdef SUPPORT_GEOIP
+#include "GeoIP.h"
+#endif
+
 
 long int ipv6calc_debug = 0;
 
+/* pipe support */
+int input_is_pipe = 0;
+#define LINEBUFFER      16384
+
 /* anonymization default values */
 int mask_ipv4 = 24;
 int mask_ipv6 = 48;
 int mask_iid = 1;
 
-#ifdef SUPPORT_IP2LOCATION
 int  use_ip2location_ipv4 = 0; /* if set to 1, IP2Location IPv4 is enabled by option(s) */
 int  use_ip2location_ipv6 = 0; /* if set to 1, IP2Location IPv6 is enabled by option(s) */
+#ifdef SUPPORT_IP2LOCATION
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV4
+char file_ip2location_ipv4[NI_MAXHOST] = IP2LOCATION_DEFAULT_FILE_IPV4;
+#else
 char file_ip2location_ipv4[NI_MAXHOST] = "";
+#endif
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV6
+char file_ip2location_ipv6[NI_MAXHOST] = IP2LOCATION_DEFAULT_FILE_IPV6;
+#else
 char file_ip2location_ipv6[NI_MAXHOST] = "";
 #endif
+#endif
 
+int  use_geoip_ipv4 = 0; /* if set to 1, GeoIP IPv4 is enabled by option(s) */
+int  use_geoip_ipv6 = 0; /* if set to 1, GeoIP IPv6 is enabled by option(s) */
 #ifdef SUPPORT_GEOIP
-int  use_geoip = 0; /* if set to 1, GeoIP is enabled by option(s) */
-#ifdef GEOIP_DEFAULT_FILE
-char file_geoip[NI_MAXHOST] = GEOIP_DEFAULT_FILE;
+#ifdef GEOIP_DEFAULT_FILE_IPV4
+char file_geoip_ipv4[NI_MAXHOST] = GEOIP_DEFAULT_FILE_IPV4;
 #else
-char file_geoip[NI_MAXHOST] = "";
+char file_geoip_ipv4[NI_MAXHOST] = "";
+#endif
+#ifdef GEOIP_DEFAULT_FILE_IPV6
+char file_geoip_ipv6[NI_MAXHOST] = GEOIP_DEFAULT_FILE_IPV6;
+#else
+char file_geoip_ipv6[NI_MAXHOST] = "";
 #endif
 #endif
 
@@ -66,10 +110,93 @@ void printversion(void) {
 #endif
 #ifdef SUPPORT_GEOIP
 	fprintf(stderr, " GeoIP");
+#ifdef SUPPORT_GEOIP_V6
+	fprintf(stderr, " GeoIPv6");
+#endif
+#endif
+#ifdef SUPPORT_DB_IEEE
+	fprintf(stderr, " DB_IEEE");
+#endif
+#ifdef SUPPORT_DB_IPV4
+	fprintf(stderr, " DB_IPV4");
+#endif
+#ifdef SUPPORT_DB_IPV6
+	fprintf(stderr, " DB_IPV6");
 #endif
 	fprintf(stderr, "\n");
 };
 
+void printversion_verbose(void) {
+	printversion();
+	fprintf(stderr, "\n");
+
+#ifdef SUPPORT_DB_IEEE
+	fprintf(stderr, "IEEE database included: %s %s\n", libieee_iab_status, libieee_oui_status);
+#else
+	fprintf(stderr, "IEEE database not included\n");
+#endif
+
+#ifdef SUPPORT_DB_IPV4
+	fprintf(stderr, "IPv4 database included: %s\n", dbipv4addr_registry_status);
+#else
+	fprintf(stderr, "IPv4 database not included\n");
+#endif
+
+#ifdef SUPPORT_DB_IPV6
+	fprintf(stderr, "IPv6 database included: %s\n", dbipv6addr_registry_status);
+#else
+	fprintf(stderr, "IPv6 database not included\n");
+#endif
+
+	fprintf(stderr, "\n");
+
+#ifdef SUPPORT_GEOIP
+#ifdef SUPPORT_GEOIP_V6
+	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 & IPv6 support\n");
+#else
+	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 support ONLY\n");
+#endif
+#ifdef SUPPORT_GEOIP_LIB_VERSION
+	fprintf(stderr, "GeoIP dynamic library version (on this system): %s\n", GeoIP_lib_version());
+#else
+	fprintf(stderr, "GeoIP dynamic library version (on this system): compiled without detection\n");
+#endif
+	if (file_geoip_ipv4 != NULL && strlen(file_geoip_ipv4) > 0) {
+		fprintf(stderr, "GeoIP IPv4 default file: %s\n", file_geoip_ipv4);
+	} else {
+		fprintf(stderr, "GeoIP IPv4 default file: not configured\n");
+	};
+#ifdef SUPPORT_GEOIP_V6
+	if (file_geoip_ipv6 != NULL && strlen(file_geoip_ipv6) > 0) {
+		fprintf(stderr, "GeoIP IPv6 default file: %s\n", file_geoip_ipv6);
+	} else {
+		fprintf(stderr, "GeoIP IPv6 default file: not configured\n");
+	};
+#endif
+#else
+	fprintf(stderr, "GeoIP support not enabled\n");
+#endif
+
+	fprintf(stderr, "\n");
+
+#ifdef SUPPORT_IP2LOCATION
+	fprintf(stderr, "IP2Location support enabled, compiled with API version: %s\n", xmakestr(API_VERSION));
+	if (file_ip2location_ipv4 != NULL && strlen(file_ip2location_ipv4) > 0) {
+		fprintf(stderr, "IP2Location IPv4 default file: %s\n", file_ip2location_ipv4);
+	} else {
+		fprintf(stderr, "IP2Location IPv4 default file: not configured\n");
+	};
+	if (file_ip2location_ipv6 != NULL && strlen(file_ip2location_ipv6) > 0) {
+		fprintf(stderr, "IP2Location IPv6 default file: %s\n", file_ip2location_ipv6);
+	} else {
+		fprintf(stderr, "IP2Location IPv6 default file: not configured\n");
+	};
+#else
+	fprintf(stderr, "IP2Location support not enabled\n");
+#endif
+
+};
+
 void printcopyright(void) {
 	fprintf(stderr, "%s\n", PROGRAM_COPYRIGHT);
 };
@@ -85,13 +212,15 @@ void printinfo(void)  {
 /**************************************************/
 /* main */
 #define DEBUG_function_name "ipv6calc/main"
-int main(int argc,char *argv[]) {
+int main(int argc, char *argv[]) {
 	char resultstring[NI_MAXHOST] = "";
 	char resultstring2[NI_MAXHOST] = "";
 	char resultstring3[NI_MAXHOST] = "";
 	int retval = 1, i, j, lop;
 	unsigned long int command = 0;
-	int bit_start = 0, bit_end = 0;
+	int bit_start = 0, bit_end = 0, force_prefix = 0;
+	char *input1 = NULL, *input2 = NULL;
+	int inputc;
 
 	/* new option style storage */	
 	uint32_t inputtype  = FORMAT_undefined;
@@ -103,7 +232,7 @@ int main(int argc,char *argv[]) {
 
 	/* used structures */
 	ipv6calc_ipv6addr ipv6addr, ipv6addr2, ipv6addr3, ipv6addr4;
-	ipv6calc_ipv4addr ipv4addr;
+	ipv6calc_ipv4addr ipv4addr, ipv4addr2;
 	ipv6calc_macaddr  macaddr;
 
 	/* clear address structures */
@@ -114,6 +243,14 @@ int main(int argc,char *argv[]) {
 	ipv4addr_clearall(&ipv4addr);
 	mac_clearall(&macaddr);
 
+	/* pipe mode */
+	char linebuffer[LINEBUFFER];
+	char token[LINEBUFFER];
+	char *charptr, *cptr, **ptrptr;
+	ptrptr = &cptr;
+	int linecounter = 0;
+	int flush_mode = 0;
+
 	if (argc <= 1) {
 		printinfo();
 		exit(EXIT_FAILURE);
@@ -124,9 +261,20 @@ int main(int argc,char *argv[]) {
 		switch (i) {
 			case -1:
 				break;
+
 			case 'v':
-				command |= CMD_printversion;
+				if ((command & CMD_printversion) != 0) {
+					// second time '-v'
+					command |= CMD_printversion_verbose;
+				} else {
+					command |= CMD_printversion;
+				};
 				break;
+
+			case 'f':
+				flush_mode = 1;
+				break;
+
 			case 'h':
 			case '?':
 				command |= CMD_printhelp;
@@ -164,30 +312,103 @@ int main(int argc,char *argv[]) {
 #endif
 				break;
 
-			case DB_geoip:
-#ifdef SUPPORT_GEOIP
-				if (ipv6calc_debug != 0) {
-					fprintf(stderr, "%s: Got GeoIP database file: %s\n", DEBUG_function_name, optarg);
+			case 'L':
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV4
+				use_ip2location_ipv4 = 1;
+#endif
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV6
+				use_ip2location_ipv6 = 1;
+#endif
+				if (use_ip2location_ipv4 == 0 && use_ip2location_ipv6 == 0) {
+					if ((formatoptions & FORMATOPTION_quiet) == 0) {
+						fprintf(stderr, " Neither support for IP2Location IPv4 or IPv6 default database is compiled in, IP2Location support disabled\n");
+					};
 				};
-				strncpy(file_geoip, optarg, sizeof(file_geoip) -1 );
-				use_geoip = 1;
+				break;
+
+			case DB_ip2location_ipv4_default:
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV4
+				use_ip2location_ipv4 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
-					fprintf(stderr, " Support for option '--db-geoip <GeoIP database file>' not compiled in, GeoIP support disabled\n");
+					fprintf(stderr, " Support for option '--db-ip2location-ipv4-default' not compiled in, IP2Location IPv4 support disabled\n");
+				};
+#endif
+				break;
+
+			case DB_ip2location_ipv6_default:
+#ifdef IP2LOCATION_DEFAULT_FILE_IPV6
+				use_ip2location_ipv6 = 1;
+#else
+				if ((formatoptions & FORMATOPTION_quiet) == 0) {
+					fprintf(stderr, " Support for option '--db-ip2location-ipv6-default' not compiled in, IP2Location IPv6 support disabled\n");
+				};
+#endif
+				break;
+
+			case DB_geoip_ipv4:
+#ifdef SUPPORT_GEOIP
+				if (ipv6calc_debug != 0) {
+					fprintf(stderr, "%s: Got GeoIP IPv4 database file: %s\n", DEBUG_function_name, optarg);
+				};
+				strncpy(file_geoip_ipv4, optarg, sizeof(file_geoip_ipv4) -1 );
+				use_geoip_ipv4 = 1;
+#else
+				if ((formatoptions & FORMATOPTION_quiet) == 0) {
+					fprintf(stderr, " Support for option '--db-geoip-ipv4 <GeoIP IPv4 database file>' not compiled in, GeoIP support disabled\n");
+				};
+#endif
+				break;
+
+			case DB_geoip_ipv6:
+#ifdef SUPPORT_GEOIP_V6
+				if (ipv6calc_debug != 0) {
+					fprintf(stderr, "%s: Got GeoIP IPv6 database file: %s\n", DEBUG_function_name, optarg);
+				};
+				strncpy(file_geoip_ipv6, optarg, sizeof(file_geoip_ipv6) -1 );
+				use_geoip_ipv6 = 1;
+#else
+				if ((formatoptions & FORMATOPTION_quiet) == 0) {
+					fprintf(stderr, " Support for option '--db-geo-ipv6 <GeoIP IPv6 database file>' not compiled in, GeoIP(v6) support disabled\n");
 				};
 #endif
 				break;
 
 			case 'G':
-#ifdef GEOIP_DEFAULT_FILE
-				use_geoip = 1;
+#ifdef GEOIP_DEFAULT_FILE_IPV4
+				use_geoip_ipv4 = 1;
+#endif
+#ifdef GEOIP_DEFAULT_FILE_IPV6
+				use_geoip_ipv6 = 1;
+#endif
+				if (use_geoip_ipv4 == 0 && use_geoip_ipv6 == 0) {
+					if ((formatoptions & FORMATOPTION_quiet) == 0) {
+						fprintf(stderr, " Neither support for GeoIP IPv4 or IPv6 default database is compiled in, GeoIP support disabled\n");
+					};
+				};
+				break;
+
+			case DB_geoip_ipv4_default:
+#ifdef GEOIP_DEFAULT_FILE_IPV4
+				use_geoip_ipv4 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
-					fprintf(stderr, " Support for option '--db-geoip-default|-G' not compiled in, GeoIP support disabled\n");
+					fprintf(stderr, " Support for option '--db-geoip-ipv4-default' not compiled in, GeoIP IPv4 support disabled\n");
 				};
 #endif
 				break;
 
+			case DB_geoip_ipv6_default:
+#ifdef GEOIP_DEFAULT_FILE_IPV6
+#ifdef SUPPORT_GEOIP_V6
+				use_geoip_ipv6 = 1;
+#endif
+#else
+				if ((formatoptions & FORMATOPTION_quiet) == 0) {
+					fprintf(stderr, " Support for option '--db-geoip-ipv6-default' not compiled in, GeoIP IPv6 support disabled\n");
+				};
+#endif
+				break;
 
 			case CMD_printexamples:
 				command = CMD_printexamples;
@@ -335,6 +556,14 @@ int main(int argc,char *argv[]) {
 				mask_iid = 1;
 				break;
 
+			case CMD_6rd_relay_prefix:
+				retval = addr_to_ipv4addrstruct(optarg, resultstring, &ipv4addr2);
+				break;
+
+			case CMD_6rd_prefix:
+				retval = addr_to_ipv6addrstruct(optarg, resultstring, &ipv6addr);
+				break;
+
 			/* format options */
 			case 'C':
 			case FORMATOPTION_NUM_printcompressed + FORMATOPTION_NUM_HEAD:
@@ -377,7 +606,19 @@ int main(int argc,char *argv[]) {
 				formatoptions |= FORMATOPTION_printuppercase;
 				break;
 				
+			case FORMATOPTION_NUM_forceprefix + FORMATOPTION_NUM_HEAD:
+				if (ipv6calc_debug != 0) fprintf(stderr, "%s: format option 'forceprefix' selected\n", DEBUG_function_name);
+				if ((atoi(optarg) >= 1) && (atoi(optarg) <= 128)) {
+					force_prefix = atoi(optarg);
+					formatoptions |= FORMATOPTION_forceprefix;
+				} else {
+					fprintf(stderr, " Argument of option 'forceprefix' is out or range (1-128): %d\n", atoi(optarg));
+					exit(EXIT_FAILURE);
+				};
+				break;
+				
 			case FORMATOPTION_NUM_printstart + FORMATOPTION_NUM_HEAD:
+				if (ipv6calc_debug != 0) fprintf(stderr, "%s: format option 'printstart' selected\n", DEBUG_function_name);
 				if ((atoi(optarg) >= 1) && (atoi(optarg) <= 128)) {
 					bit_start = atoi(optarg);
 					formatoptions |= FORMATOPTION_printstart;
@@ -388,6 +629,7 @@ int main(int argc,char *argv[]) {
 				break;
 				
 			case FORMATOPTION_NUM_printend + FORMATOPTION_NUM_HEAD:
+				if (ipv6calc_debug != 0) fprintf(stderr, "%s: format option 'printend' selected\n", DEBUG_function_name);
 				if ((atoi(optarg) >= 1) && (atoi(optarg) <= 128)) {
 					bit_end = atoi(optarg);
 					formatoptions |= FORMATOPTION_printend;
@@ -469,7 +711,7 @@ int main(int argc,char *argv[]) {
 			default:
 				fprintf(stderr, "Usage: (see '%s --command -?|-h|--help' for more help)\n", PROGRAM_NAME);
 				printhelp();
-				break;
+				exit(EXIT_FAILURE);
 		};
 	};
 	argv += optind;
@@ -503,12 +745,16 @@ int main(int argc,char *argv[]) {
 	};
 
 	if (ipv6calc_debug != 0) {
-		fprintf(stderr, "Debug value:%lx  command:%lx  inputtype:%lx   outputtype:%lx  action:%lx  formatoptions:%x\n", (unsigned long) ipv6calc_debug, command, (unsigned long) inputtype, (unsigned long) outputtype, (unsigned long) action, (unsigned) formatoptions); 
+		fprintf(stderr, "Debug value:%08lx command:%08lx inputtype:%08lx outputtype:%08lx action:%08lx formatoptions:%08lx\n", (unsigned long) ipv6calc_debug, command, (unsigned long) inputtype, (unsigned long) outputtype, (unsigned long) action, (unsigned long) formatoptions); 
 	};
 	
 	/* do work depending on selection */
-	if (command == CMD_printversion) {
-		printversion();
+	if ((command & CMD_printversion) != 0) {
+		if ((command & CMD_printversion_verbose) != 0) {
+			printversion_verbose();
+		} else {
+			printversion();
+		};
 		exit(EXIT_FAILURE);
 	};
 
@@ -517,8 +763,19 @@ int main(int argc,char *argv[]) {
 		exit(EXIT_FAILURE);
 	};
 	
-	if (ipv6calc_debug != 0) {
-		fprintf(stderr, "%s: Got input %s\n", DEBUG_function_name, argv[0]);
+	if (argc > 0) {
+		if (ipv6calc_debug != 0) {
+			fprintf(stderr, "%s: Got input %s\n", DEBUG_function_name, argv[0]);
+		};
+	} else {
+		if (isatty (STDIN_FILENO)) {
+		} else {
+			input_is_pipe = 1;
+			if (ipv6calc_debug != 0) {
+				fprintf(stderr, "%s: Input is a pipe\n", DEBUG_function_name);
+			};
+			formatoptions |= FORMATOPTION_quiet; // force quiet mode
+		};
 	};
 
 	/***** automatic action handling *****/
@@ -568,6 +825,10 @@ int main(int argc,char *argv[]) {
 
 	/* check length of input */
 	if (argc > 0) {
+		if (input_is_pipe == 1) {
+			fprintf(stderr, "Additional input found in pipe mode\n");
+			exit(EXIT_FAILURE);
+		};
 		for (i = 0; i < argc; i++) {
 			if ( argv[i] != NULL ) {
 				if ( strlen(argv[i]) >= NI_MAXHOST ) {
@@ -578,9 +839,93 @@ int main(int argc,char *argv[]) {
 			};
 		};
 	};
+
+	/* loop for pipe */
+	if (input_is_pipe == 1) {
+PIPE_input:
+                /* read line from stdin */
+                charptr = fgets(linebuffer, LINEBUFFER, stdin);
+
+		if (charptr == NULL) {
+			/* end of input */
+			exit(retval);
+		};
+
+		linecounter++;
+
+		if (linecounter == 1) {
+			if (ipv6calc_debug == 1) {
+				fprintf(stderr, "Ok, proceeding stdin...\n");
+			};
+		};
+		
+		if (ipv6calc_debug == 1) {
+			fprintf(stderr, "Line: %d\r", linecounter);
+		};
+
+		if (strlen(linebuffer) >= NI_MAXHOST) {
+			fprintf(stderr, "Line too long: %d\n", linecounter);
+			exit(EXIT_FAILURE);
+		};
+		
+		if (strlen(linebuffer) == 0) {
+			fprintf(stderr, "Line empty: %d\n", linecounter);
+			goto PIPE_input;
+		};
+		
+		if (ipv6calc_debug != 0) {
+			fprintf(stderr, "%s: stdin got line: '%s'\n", DEBUG_function_name, linebuffer);
+		};
+
+		if (linebuffer[strlen(linebuffer) - 1] == '\n') {
+			/* remove trailing \n */
+			linebuffer[strlen(linebuffer) - 1] = '\0';
+		};
+
+		/* look for first token */
+		charptr = strtok_r(linebuffer, " \t\n", ptrptr);
+
+		if ( charptr == NULL ) {
+			fprintf(stderr, "Line contains no token: %d\n", linecounter);
+			goto PIPE_input;
+		};
+
+		if ( strlen(charptr) >=  LINEBUFFER) {
+			fprintf(stderr, "Line too strange: %d\n", linecounter);
+			goto PIPE_input;
+		};
+
+		snprintf(token, sizeof(token) - 1, "%s", charptr);
+		
+		input1 = token;
+		inputc = 1;
+
+		if (ipv6calc_debug != 0) {
+			fprintf(stderr, "%s: Token 1: '%s'\n", DEBUG_function_name, input1);
+		};
+
+		/* check for second token */
+		if (*ptrptr[0] != '\0') {
+			input2 = *ptrptr;
+			inputc = 2;
+
+			if (ipv6calc_debug != 0) {
+				fprintf(stderr, "%s: Token 2: '%s'\n", DEBUG_function_name, input2);
+			};
+		};
+
+	} else {
+		inputc = argc;
+		if (argc >= 1) {
+			input1 = argv[0];
+		};
+		if (argc >= 2) {
+			input2 = argv[1];
+		};
+	};
 	
 	/* autodetection */
-	if ((inputtype == FORMAT_undefined || inputtype == FORMAT_auto) && argc > 0) {
+	if ((inputtype == FORMAT_undefined || inputtype == FORMAT_auto) && inputc > 0) {
 		if (ipv6calc_debug != 0) {
 			fprintf(stderr, "%s: Call input type autodetection\n", DEBUG_function_name);
 		};
@@ -589,7 +934,7 @@ int main(int argc,char *argv[]) {
 			fprintf(stderr, "No input type specified, try autodetection...");
 		};
 		
-		inputtype = libipv6calc_autodetectinput(argv[0]);
+		inputtype = libipv6calc_autodetectinput(input1);
 
 		if ( inputtype != FORMAT_undefined ) {
 			for (i = 0; i < (int) (sizeof(ipv6calc_formatstrings) / sizeof(ipv6calc_formatstrings[0])); i++) {
@@ -641,90 +986,100 @@ int main(int argc,char *argv[]) {
 	/* proceed input depending on type */	
 	retval = -1; /* default */
 
+	if (ipv6calc_debug != 0) {
+		fprintf(stderr, "%s: Process input (inputtype:%lx, count=%d)\n", DEBUG_function_name, (unsigned long) inputtype, inputc);
+	};
+
 	switch (inputtype) {
 		case FORMAT_ipv6addr:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = addr_to_ipv6addrstruct(argv[0], resultstring, &ipv6addr);
+		case FORMAT_ipv4addr:
+		case FORMAT_ipv4hex:
+		case FORMAT_ipv4revhex:
+		case FORMAT_base85:
+		case FORMAT_mac:
+		case FORMAT_ifinet6:
+		case FORMAT_revnibbles_int:
+		case FORMAT_revnibbles_arpa:
+		case FORMAT_bitstring:
+			if (inputc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
+			break;
+
+		case FORMAT_iid_token:
+		case FORMAT_prefix_mac:
+			if (inputc < 2) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
+			break;
+
+		case FORMAT_auto_noresult:
+		default:
+			fprintf(stderr, " Input-type isn't autodetected\n");
+			exit(EXIT_FAILURE);
+	};
+
+	switch (inputtype) {
+		case FORMAT_ipv6addr:
+			retval = addr_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
 			argc--;
 			break;
 
 		case FORMAT_ipv4addr:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = addr_to_ipv4addrstruct(argv[0], resultstring, &ipv4addr);
+			retval = addr_to_ipv4addrstruct(input1, resultstring, &ipv4addr);
 			argc--;
 			break;
 			
 		case FORMAT_ipv4hex:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = addrhex_to_ipv4addrstruct(argv[0], resultstring, &ipv4addr, 0);
+			retval = addrhex_to_ipv4addrstruct(input1, resultstring, &ipv4addr, 0);
 			argc--;
 			break;
 
 		case FORMAT_ipv4revhex:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = addrhex_to_ipv4addrstruct(argv[0], resultstring, &ipv4addr, 1);
+			retval = addrhex_to_ipv4addrstruct(input1, resultstring, &ipv4addr, 1);
 			argc--;
 			break;
 
 		case FORMAT_base85:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = base85_to_ipv6addrstruct(argv[0], resultstring, &ipv6addr);
+			retval = base85_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
 			argc--;
 			break;
 			
 		case FORMAT_mac:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = mac_to_macaddrstruct(argv[0], resultstring, &macaddr);
+			retval = mac_to_macaddrstruct(input1, resultstring, &macaddr);
 			argc--;
 			break;
 
 		case FORMAT_ifinet6:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			if (argc < 2) {
-				retval = libifinet6_ifinet6_to_ipv6addrstruct(argv[0], resultstring, &ipv6addr);
-				argc--;
+			if (inputc < 2) {
+				retval = libifinet6_ifinet6_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
 			} else {
-				retval = libifinet6_ifinet6_withprefixlength_to_ipv6addrstruct(argv[0], argv[1], resultstring, &ipv6addr);
-				argc -= 2;
+				retval = libifinet6_ifinet6_withprefixlength_to_ipv6addrstruct(input1, input2, resultstring, &ipv6addr);
 			};
 			break;
 
 		case FORMAT_iid_token:
 			/* Get first interface identifier */
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = identifier_to_ipv6addrstruct(argv[0], resultstring, &ipv6addr);
-			argc--;
+			retval = identifier_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
 			if (retval != 0) { break; };
 			
 			/* Get second token */
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = tokenlsb64_to_ipv6addrstruct(argv[1], resultstring, &ipv6addr2);
-			argc--;
+			retval = tokenlsb64_to_ipv6addrstruct(input2, resultstring, &ipv6addr2);
 			break;
 			
 		case FORMAT_prefix_mac:
 			/* Get first: IPv6 prefix */
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = addr_to_ipv6addrstruct(argv[0], resultstring, &ipv6addr);
-			argc--;
+			retval = addr_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
 			if (retval != 0) { break; };
 			
 			/* Get second: MAC address */
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = mac_to_macaddrstruct(argv[1], resultstring, &macaddr);
-			argc--;
+			retval = mac_to_macaddrstruct(input2, resultstring, &macaddr);
 			break;
 
 		case FORMAT_revnibbles_int:
 		case FORMAT_revnibbles_arpa:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = librfc1886_nibblestring_to_ipv6addrstruct(argv[0], &ipv6addr, resultstring);
+			retval = librfc1886_nibblestring_to_ipv6addrstruct(input1, &ipv6addr, resultstring);
 			argc--;
 			break;
 			
 		case FORMAT_bitstring:
-			if (argc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
-			retval = librfc2874_bitstring_to_ipv6addrstruct(argv[0], &ipv6addr, resultstring);
+			retval = librfc2874_bitstring_to_ipv6addrstruct(input1, &ipv6addr, resultstring);
 			argc--;
 			break;
 			
@@ -753,6 +1108,12 @@ int main(int argc,char *argv[]) {
 	};
 
 	if (ipv6addr.flag_valid == 1) {
+		/* force prefix */
+		if ((formatoptions & (FORMATOPTION_forceprefix)) != 0) {
+			ipv6addr.flag_prefixuse = 1;
+			ipv6addr.prefixlength = force_prefix;
+		};
+		
 		/* mask bits */
 		if ((formatoptions & (FORMATOPTION_maskprefix | FORMATOPTION_masksuffix)) != 0) {
 			if (ipv6addr.flag_prefixuse == 1) {
@@ -789,7 +1150,7 @@ int main(int argc,char *argv[]) {
 			/* default */
 			ipv6addr.bit_end = 128;
 		};
-		
+
 		/* prefix+suffix */
 		if ((formatoptions & (FORMATOPTION_printprefix | FORMATOPTION_printsuffix)) != 0) {
 			if ( ipv6addr.flag_prefixuse == 0 ) {
@@ -886,6 +1247,8 @@ int main(int argc,char *argv[]) {
 			outputtype = FORMAT_mac;
 		} else if ( (inputtype == FORMAT_bitstring) || (inputtype == FORMAT_base85) ) {
 			outputtype = FORMAT_ipv6addr;
+		} else if ( (inputtype == FORMAT_ipv4addr) && (action == ACTION_6rd_local_prefix) ) {
+			outputtype = FORMAT_ipv6addr;
 		};
 
 		if ( outputtype != FORMAT_undefined ) {
@@ -906,14 +1269,17 @@ int main(int argc,char *argv[]) {
 
 	if ( (outputtype & (FORMAT_revnibbles_int | FORMAT_revnibbles_arpa)) != 0 ) {
 		/* workaround for reverse IPv4 */
-		if (ipv4addr.flag_valid == 1) {
+		 if (ipv6addr.flag_valid == 0 && ipv4addr.flag_valid == 1) {
 			outputtype = FORMAT_revipv4;
 		};
 	};
 
-	
 	/* clear resultstring */
 	snprintf(resultstring, sizeof(resultstring) - 1, "%s", "");
+
+	if (ipv6calc_debug != 0) {
+		fprintf(stderr, "%s: Process action (action:%lx)\n", DEBUG_function_name, (unsigned long) action);
+	};
 	
 	switch (action) {
 		case ACTION_mac_to_eui64:
@@ -1006,6 +1372,54 @@ int main(int argc,char *argv[]) {
 			};
 			break;
 
+		case ACTION_6rd_local_prefix:
+			/* check formats */
+			if ( outputtype != FORMAT_ipv6addr ) {
+				fprintf(stderr, "output type incompatible (no IPv6 address)!\n");
+				exit(EXIT_FAILURE);
+			};
+	
+			if ( inputtype != FORMAT_ipv4addr ) {
+				fprintf(stderr, "input type incompatible (no IPv4 address)!\n");
+				exit(EXIT_FAILURE);
+			};
+	
+			/* check IPv6 prefix */
+			if ( ipv6addr.flag_valid != 1 ) {
+				fprintf(stderr, "No valid IPv6 address given as 6rd prefix!\n");
+				exit(EXIT_FAILURE);
+			}
+
+			if ( !ipv6addr.flag_prefixuse ) {
+				fprintf(stderr, "Specified IPv6 address has no prefix length!\n");
+				exit(EXIT_FAILURE);
+			}
+
+			/* check IPv4 local ip */
+			if ( ipv4addr.flag_valid != 1 ) {
+				fprintf(stderr, "No valid IPv4 address given as local IP!\n");
+				exit(EXIT_FAILURE);
+			}
+		
+			if ( ipv4addr.flag_prefixuse ) {
+				fprintf(stderr, "No prefix length should be specified for the local IP!\n");
+				exit(EXIT_FAILURE);
+			}
+
+			/* check IPv4 relay prefix */
+			if ( ipv4addr2.flag_valid != 1 ) {
+				fprintf(stderr, "No valid IPv4 address given as 6rd relay prefix!\n");
+				exit(EXIT_FAILURE);
+			}
+
+			if ( !ipv4addr2.flag_prefixuse ) {
+				fprintf(stderr, "Specified relay prefix IPv4 address has no prefix length!\n");
+				exit(EXIT_FAILURE);
+			}
+
+			retval = librfc5569_calc_6rd_local_prefix(&ipv6addr, &ipv4addr2, &ipv4addr, resultstring);
+			break;
+
 		case ACTION_undefined:
 			/* no action selected */
 			break;
@@ -1027,7 +1441,7 @@ int main(int argc,char *argv[]) {
 
 	/***** output type *****/
 	if (ipv6calc_debug != 0) {
-		fprintf(stderr, "%s: Start of output handling\n", DEBUG_function_name);
+		fprintf(stderr, "%s: Process output (outputtype:%lx)\n", DEBUG_function_name, (unsigned long) outputtype);
 	};
 	
 	/* catch showinfo */	
@@ -1049,18 +1463,19 @@ int main(int argc,char *argv[]) {
 		goto RESULT_print;
 	};
 
-
 	switch (outputtype) {
 		case -1:
 			/* old implementation */
 			break;
 
 		case FORMAT_base85:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_base85\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1) { fprintf(stderr, "No valid IPv6 address given!\n"); exit(EXIT_FAILURE); };
 			retval = ipv6addrstruct_to_base85(&ipv6addr, resultstring);
 			break;
 				
 		case FORMAT_bitstring:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_bitstring\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1) { fprintf(stderr, "No valid IPv6 address given!\n"); exit(EXIT_FAILURE); };
 			if (ipv6calc_debug != 0) {
 				fprintf(stderr, "%s: Call 'librfc2874_addr_to_bitstring'\n", DEBUG_function_name);
@@ -1069,6 +1484,7 @@ int main(int argc,char *argv[]) {
 			break;
 			
 		case FORMAT_octal:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_octal\n", DEBUG_function_name); }
 			if (ipv4addr.flag_valid == 1) {
 				retval = libipv4addr_to_octal(&ipv4addr, resultstring, formatoptions);
 			} else if (ipv6addr.flag_valid == 1) {
@@ -1077,6 +1493,7 @@ int main(int argc,char *argv[]) {
 			break;
 				
 		case FORMAT_hex:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_hex\n", DEBUG_function_name); }
 			if (ipv4addr.flag_valid == 1) {
 				retval = libipv4addr_to_hex(&ipv4addr, resultstring, formatoptions);
 			} else if (ipv6addr.flag_valid == 1) {
@@ -1086,6 +1503,7 @@ int main(int argc,char *argv[]) {
 				
 		case FORMAT_revnibbles_int:
 		case FORMAT_revnibbles_arpa:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_revnibbles_int/arpa\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid == 1) {
 				switch (outputtype) {
 					case FORMAT_revnibbles_int:
@@ -1101,6 +1519,7 @@ int main(int argc,char *argv[]) {
 			break;
 				
 		case FORMAT_revipv4:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_revipv4\n", DEBUG_function_name); }
 			if (ipv4addr.flag_valid == 1) {
 				retval = libipv4addr_to_reversestring(&ipv4addr, resultstring, formatoptions);
 			} else {
@@ -1109,11 +1528,13 @@ int main(int argc,char *argv[]) {
 			break;
 			
 		case FORMAT_ifinet6:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_ifinet6\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1) { fprintf(stderr, "No valid IPv6 address given!\n"); exit(EXIT_FAILURE); };
 			retval = libifinet6_ipv6addrstruct_to_ifinet6(&ipv6addr, resultstring);
 			break;
 
 		case FORMAT_ipv6addr:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_ipv6addr\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1) { fprintf(stderr, "No valid IPv6 address given!\n"); exit(EXIT_FAILURE); };
 			if ((formatoptions & (FORMATOPTION_printuncompressed | FORMATOPTION_printfulluncompressed | FORMATOPTION_printprefix | FORMATOPTION_printsuffix)) != 0) {
 				retval = libipv6addr_ipv6addrstruct_to_uncompaddr(&ipv6addr, resultstring, formatoptions);
@@ -1123,22 +1544,26 @@ int main(int argc,char *argv[]) {
 			break;
 			
 		case FORMAT_ipv4addr:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_ipv4addr\n", DEBUG_function_name); }
 			if (ipv4addr.flag_valid != 1) { fprintf(stderr, "No valid IPv4 address given!\n"); exit(EXIT_FAILURE); };
 			retval = libipv4addr_ipv4addrstruct_to_string(&ipv4addr, resultstring, formatoptions);
 			break;
 			
 		case FORMAT_eui64:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_eui64\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1) { fprintf(stderr, "No valid IPv6 address given!\n"); exit(EXIT_FAILURE); };
 			formatoptions |= FORMATOPTION_printsuffix;
 			retval = libipv6addr_ipv6addrstruct_to_uncompaddr(&ipv6addr, resultstring, formatoptions);
 			break;
 			
 		case FORMAT_mac:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_mac\n", DEBUG_function_name); }
 			if (macaddr.flag_valid != 1) { fprintf(stderr, "No valid MAC address given!\n"); exit(EXIT_FAILURE); };
 			retval = macaddrstruct_to_string(&macaddr, resultstring, formatoptions);
 			break;
 			
 		case FORMAT_iid_token:
+			if (ipv6calc_debug != 0) { fprintf(stderr, "%s: Start of output handling for FORMAT_iid_token\n", DEBUG_function_name); }
 			if (ipv6addr.flag_valid != 1 || ipv6addr2.flag_valid != 1) { fprintf(stderr, "No valid IPv6 addresses given!\n"); exit(EXIT_FAILURE); };
 			/* get interface identifier */
 			retval = libipv6addr_ipv6addrstruct_to_uncompaddr(&ipv6addr, resultstring2, formatoptions | FORMATOPTION_printsuffix);
@@ -1153,7 +1578,7 @@ int main(int argc,char *argv[]) {
 
 
 		default:
-			fprintf(stderr, " Outputtype isn't implemented\n");
+			fprintf(stderr, " Outputtype isn't implemented: %lx\n", (unsigned long) outputtype);
 			exit(EXIT_FAILURE);
 	};
 
@@ -1170,6 +1595,14 @@ RESULT_print:
 			fprintf(stderr, "%s\n", resultstring);
 		};
 	};
+
+	if (input_is_pipe == 1) {
+		if (flush_mode == 1) {
+			fflush(stdout);
+		};
+		goto PIPE_input;
+	};
+
 	exit(retval);
 };
 #undef DEBUG_function_name
