@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc
  * File       : showinfo.c
- * Version    : $Id: showinfo.c,v 1.44 2011/03/29 18:33:32 peter Exp $
+ * Version    : $Id: showinfo.c,v 1.48 2011/10/06 19:22:47 peter Exp $
  * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
@@ -45,8 +45,10 @@ extern char file_ip2location_ipv6[NI_MAXHOST];
 #ifdef SUPPORT_GEOIP
 #include "GeoIP.h"
 #include "GeoIPCity.h"
-extern int use_geoip;
-extern char file_geoip[NI_MAXHOST];
+extern int use_geoip_ipv4;
+extern int use_geoip_ipv6;
+extern char file_geoip_ipv4[NI_MAXHOST];
+extern char file_geoip_ipv6[NI_MAXHOST];
 #endif
 
 
@@ -95,6 +97,17 @@ void showinfo_availabletypes(void) {
 	fprintf(stderr, " IP2LOCATION_ZIPCODE=...       : Zip code of IP address\n");
 	fprintf(stderr, " IP2LOCATION_DATABASE_INFO_IPV4=... : Information about the used IPv4 database\n");
 	fprintf(stderr, " IP2LOCATION_DATABASE_INFO_IPV6=... : Information about the used IPv6 database\n");
+#endif
+#ifdef SUPPORT_GEOIP
+	fprintf(stderr, " GEOIP_COUNTRY_SHORT=...       : Country code of IP address\n");
+	fprintf(stderr, " GEOIP_COUNTRY_LONG=...        : Country of IP address\n");
+	fprintf(stderr, " GEOIP_REGION=...              : Region of IP address\n");
+	fprintf(stderr, " GEOIP_CITY=...                : City of IP address\n");
+	fprintf(stderr, " GEOIP_ZIPCODE=...             : Zip code of IP address\n");
+	fprintf(stderr, " GEOIP_LATITUDE=...            : Latitude of IP address\n");
+	fprintf(stderr, " GEOIP_LONGITUDE=...           : Longitude of IP address\n");
+	fprintf(stderr, " GEOIP_DATABASE_INFO_IPV4=...  : Information about the used IPv4 database\n");
+	fprintf(stderr, " GEOIP_DATABASE_INFO_IPV6=...  : Information about the used IPv6 database\n");
 #endif
 	fprintf(stderr, " IPV6CALC_VERSION=x.y          : Version of ipv6calc\n");
 	fprintf(stderr, " IPV6CALC_COPYRIGHT=\"...\"      : Copyright string\n");
@@ -146,7 +159,7 @@ static void print_ip2location(const char *addrstring, const uint32_t formatoptio
 	} else if (version == 6) {
 		file_ip2location = file_ip2location_ipv6;
 	} else {
-		fprintf(stderr, " IP2Location version %d not supported\n", version);
+		fprintf(stderr, " IP2Location for IP version %d not supported\n", version);
 		return;
 	};
 
@@ -164,7 +177,7 @@ static void print_ip2location(const char *addrstring, const uint32_t formatoptio
 
 	if (IP2LocationObj == NULL) {
 		/* error on opening database, nothing more todo */
-		fprintf(stderr, " IP2Location can't open database\n");
+		fprintf(stderr, "%s: IP2Location can't open database: %s\n", DEBUG_function_name, file_ip2location);
 		return;
 	};
 
@@ -243,7 +256,9 @@ static void print_ip2location(const char *addrstring, const uint32_t formatoptio
 #ifdef SUPPORT_GEOIP
 #define DEBUG_function_name "showinfo/print_geoip"
 /* print GeoIP information */
-static void print_geoip(const char *addrstring, const uint32_t formatoptions, const char *additionalstring) {
+static void print_geoip(const char *addrstring, const uint32_t formatoptions, const char *additionalstring, int version) {
+
+	char *file_geoip;
 
 	const char *returnedCountry = NULL;
 	const char *returnedCountryName = NULL;
@@ -254,32 +269,41 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 	uint32_t machinereadable = (formatoptions & FORMATOPTION_machinereadable);
 	char tempstring[NI_MAXHOST] = "";
 
+	if (version == 4) {
+		file_geoip = file_geoip_ipv4;
+	} else if (version == 6) {
+		file_geoip = file_geoip_ipv6;
+	} else {
+		fprintf(stderr, " GeoIP for IPv%d not supported\n", version);
+		return;
+	};
+
 	if (strlen(file_geoip) == 0) {
 		/* no file given, nothing more todo */
-		fprintf(stderr, " GeoIP database file not given\n");
+		fprintf(stderr, " GeoIP IPv%d database file not given\n", version);
 	};
 
 	if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
-		fprintf(stderr, "%s: GeoIP try to open database: %s\n", DEBUG_function_name, file_geoip);
+		fprintf(stderr, "%s: GeoIP IPv%d try to open database: %s\n", DEBUG_function_name, version, file_geoip);
 	};
 
 	GeoIP *gi = GeoIP_open(file_geoip, GEOIP_STANDARD);
 
 	if (gi == NULL) {
 		/* error on opening database, nothing more todo */
-		fprintf(stderr, " GeoIP can't open database\n");
+		fprintf(stderr, " GeoIP IPv%d can't open database: %s\n", version, file_geoip);
 		return;
 	};
 
 	if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
-		fprintf(stderr, "%s: GeoIP database opened: %s\n", DEBUG_function_name, file_geoip);
+		fprintf(stderr, "%s: GeoIP IPv%d database opened: %s\n", DEBUG_function_name, version, file_geoip);
 	};
 
 	if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
-		fprintf(stderr, "%s: GeoIP check available databases\n", DEBUG_function_name);
+		fprintf(stderr, "%s: GeoIP IPv%d check available databases\n", DEBUG_function_name, version);
 	};
 
-	if (GeoIP_database_edition(gi) == GEOIP_COUNTRY_EDITION) {
+	if (GeoIP_database_edition(gi) == GEOIP_COUNTRY_EDITION && version == 4) {
 		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
 			fprintf(stderr, "%s: GeoIP country database available\n", DEBUG_function_name);
 		};
@@ -288,7 +312,18 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 		returnedCountryName = GeoIP_country_name_by_addr(gi, addrstring);
 	};
 
-	if (GeoIP_database_edition(gi) == GEOIP_CITY_EDITION_REV1) {
+#ifdef SUPPORT_GEOIP_V6
+	if (GeoIP_database_edition(gi) == GEOIP_COUNTRY_EDITION_V6 && version == 6) {
+		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
+			fprintf(stderr, "%s: GeoIP country database IPv6 available\n", DEBUG_function_name);
+		};
+
+		returnedCountry = GeoIP_country_code_by_addr_v6(gi, addrstring);
+		returnedCountryName = GeoIP_country_name_by_addr_v6(gi, addrstring);
+	};
+#endif
+
+	if (GeoIP_database_edition(gi) == GEOIP_CITY_EDITION_REV1 && version == 4) {
 		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
 			fprintf(stderr, "%s: GeoIP city database available\n", DEBUG_function_name);
 		};
@@ -299,7 +334,7 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 		flag_geoip_ok = 1;
 
 		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
-			fprintf(stderr, "%s: GeoIP city database result\n", DEBUG_function_name);
+			fprintf(stderr, "%s: GeoIP IPv%d city database result\n", DEBUG_function_name, version);
 		};
 
 		if ( machinereadable != 0 ) {
@@ -332,7 +367,7 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 		flag_geoip_ok = 1;
 
 		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
-			fprintf(stderr, "%s: GeoIP city database result\n", DEBUG_function_name);
+			fprintf(stderr, "%s: GeoIP IP%d contry database result\n", DEBUG_function_name, version);
 		};
 
 		if ( machinereadable != 0 ) {
@@ -353,12 +388,12 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 			flag_geoip_info_shown = 1;
 
 			if ( machinereadable != 0 ) {
-				snprintf(tempstring, sizeof(tempstring) - 1, "GEOIP_DATABASE_INFO=%s apiversion=system", GeoIP_database_info(gi));
+				snprintf(tempstring, sizeof(tempstring) - 1, "GEOIP_DATABASE_INFO_IPV%d=%s apiversion=system", version, GeoIP_database_info(gi));
 				printout(tempstring);
 			};
 		};
 	} else {
-		if ((formatoptions & FORMATOPTION_quiet) == 0) {
+		if ( (ipv6calc_debug & DEBUG_showinfo) != 0 ) {
 			fprintf(stderr, "%s: GeoIP returned no record for address: %s\n", DEBUG_function_name, addrstring);
 		};
 	};
@@ -435,8 +470,8 @@ static void print_ipv4addr(const ipv6calc_ipv4addr *ipv4addrp, const uint32_t fo
 
 #ifdef SUPPORT_GEOIP
 	/* GeoIP information */
-	if (use_geoip == 1) {
-		print_geoip(tempipv4string, formatoptions, embeddedipv4string);
+	if (use_geoip_ipv4 == 1) {
+		print_geoip(tempipv4string, formatoptions, embeddedipv4string, 4);
 	};
 #endif
 
@@ -519,7 +554,7 @@ static void print_eui48(const ipv6calc_macaddr *macaddrp, const uint32_t formato
 	if ( (macaddrp->addr[0] == 0xfc) && (macaddrp->addr[1] == 0xfc) ) {
 		/* copy address */
 		for ( i = 0; i <= 3; i++ ) {
-			ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) macaddrp->addr[i + 2]);
+			ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) macaddrp->addr[i + 2]);
 		};
 
 		if ( machinereadable != 0 ) {
@@ -676,7 +711,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 
 	if ( (typeinfo & IPV6_NEW_ADDR_6TO4) != 0 ) {
 		for (i = 0; i <= 3; i++) {
-			ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) 2 + i));
+			ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) 2 + i));
 		};
 
 		retval = libipv4addr_ipv4addrstruct_to_string(&ipv4addr, helpstring, 0);
@@ -703,12 +738,12 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 	if ( (typeinfo & IPV6_NEW_ADDR_TEREDO) != 0 ) {
 		/* extract Teredo client IPv4 address */
 		for (i = 0; i <= 3; i++) {
-			ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) 12 + i) ^ 0xff);
+			ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) 12 + i) ^ 0xff);
 		};
 
 		/* extract Teredo server IPv4 address */
 		for (i = 0; i <= 3; i++) {
-			ipv4addr_setoctett(&ipv4addr2, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) 4 + i));
+			ipv4addr_setoctet(&ipv4addr2, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) 4 + i));
 		};
 
 		print_ipv4addr(&ipv4addr, formatoptions | FORMATOPTION_printembedded, "TEREDO-CLIENT");
@@ -722,7 +757,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 		};	
 
 		/* extract Teredo client UDP port */
-		port = (uint16_t) (((uint16_t) ipv6addr_getoctett(ipv6addrp, (unsigned int) 10) << 8 | (uint16_t) ipv6addr_getoctett(ipv6addrp, (unsigned int) 11)) ^ 0xffff);
+		port = (uint16_t) (((uint16_t) ipv6addr_getoctet(ipv6addrp, (unsigned int) 10) << 8 | (uint16_t) ipv6addr_getoctet(ipv6addrp, (unsigned int) 11)) ^ 0xffff);
 
 		if ( machinereadable != 0 ) {
 			snprintf(tempstring, sizeof(tempstring) - 1, "TEREDO_PORT_CLIENT=%u", (unsigned int) port);
@@ -742,7 +777,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 
 	if ( (typeinfo & IPV6_NEW_ADDR_NAT64) != 0 )  {
 		for (i = 0; i <= 3; i++) {
-			ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) (i + 12)));
+			ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
 		};
 
 		if ( machinereadable != 0 ) {
@@ -804,7 +839,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 			fprintf(stdout, "Address type is compat/mapped and include an IPv4 address\n");
 		};
 		for (i = 0; i <= 3; i++) {
-			ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) (i + 12)));
+			ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
 		};
 		print_ipv4addr(&ipv4addr, formatoptions | FORMATOPTION_printembedded, "COMPAT/MAPPED");
 	};
@@ -818,38 +853,38 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 			fprintf(stdout, "Interface identifier: %04x:%04x:%04x:%04x\n", (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addr_getword(ipv6addrp, 6), (unsigned int) ipv6addr_getword(ipv6addrp, 7));
 		};
 
-		if (ipv6addr_getoctett(ipv6addrp, 11) == 0xff && ipv6addr_getoctett(ipv6addrp, 12) == 0xfe) {
+		if (ipv6addr_getoctet(ipv6addrp, 11) == 0xff && ipv6addr_getoctet(ipv6addrp, 12) == 0xfe) {
 			/* EUI-48 */
-			macaddr.addr[0] = ipv6addr_getoctett(ipv6addrp,  8) ^ 0x02;
-			macaddr.addr[1] = ipv6addr_getoctett(ipv6addrp,  9);
-			macaddr.addr[2] = ipv6addr_getoctett(ipv6addrp, 10);
-			macaddr.addr[3] = ipv6addr_getoctett(ipv6addrp, 13);
-			macaddr.addr[4] = ipv6addr_getoctett(ipv6addrp, 14);
-			macaddr.addr[5] = ipv6addr_getoctett(ipv6addrp, 15);
+			macaddr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
+			macaddr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
+			macaddr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
+			macaddr.addr[3] = ipv6addr_getoctet(ipv6addrp, 13);
+			macaddr.addr[4] = ipv6addr_getoctet(ipv6addrp, 14);
+			macaddr.addr[5] = ipv6addr_getoctet(ipv6addrp, 15);
 			print_eui48(&macaddr, formatoptions);
 		} else {
 			/* Check for global EUI-64 */
-			if ( (ipv6addr_getoctett(ipv6addrp, 8) & 0x02) != 0 ) {
-				eui64addr.addr[0] = ipv6addr_getoctett(ipv6addrp,  8) ^ 0x02;
-				eui64addr.addr[1] = ipv6addr_getoctett(ipv6addrp,  9);
-				eui64addr.addr[2] = ipv6addr_getoctett(ipv6addrp, 10);
-				eui64addr.addr[3] = ipv6addr_getoctett(ipv6addrp, 11);
-				eui64addr.addr[4] = ipv6addr_getoctett(ipv6addrp, 12);
-				eui64addr.addr[5] = ipv6addr_getoctett(ipv6addrp, 13);
-				eui64addr.addr[6] = ipv6addr_getoctett(ipv6addrp, 14);
-				eui64addr.addr[7] = ipv6addr_getoctett(ipv6addrp, 15);
+			if ( (ipv6addr_getoctet(ipv6addrp, 8) & 0x02) != 0 ) {
+				eui64addr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
+				eui64addr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
+				eui64addr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
+				eui64addr.addr[3] = ipv6addr_getoctet(ipv6addrp, 11);
+				eui64addr.addr[4] = ipv6addr_getoctet(ipv6addrp, 12);
+				eui64addr.addr[5] = ipv6addr_getoctet(ipv6addrp, 13);
+				eui64addr.addr[6] = ipv6addr_getoctet(ipv6addrp, 14);
+				eui64addr.addr[7] = ipv6addr_getoctet(ipv6addrp, 15);
 				print_eui64(&eui64addr, formatoptions);
 			} else {
 				if ( (typeinfo & IPV6_NEW_ADDR_SOLICITED_NODE) != 0 ) {
 					if ( machinereadable != 0 ) {
-						snprintf(tempstring, sizeof(tempstring) - 1, "EUI64=??:??:??:??:??:%02x:%02x:%02x", (unsigned int) ipv6addr_getoctett(ipv6addrp, 13), (unsigned int) ipv6addr_getoctett(ipv6addrp, 14), (unsigned int) ipv6addr_getoctett(ipv6addrp, 15));
+						snprintf(tempstring, sizeof(tempstring) - 1, "EUI64=??:??:??:??:??:%02x:%02x:%02x", (unsigned int) ipv6addr_getoctet(ipv6addrp, 13), (unsigned int) ipv6addr_getoctet(ipv6addrp, 14), (unsigned int) ipv6addr_getoctet(ipv6addrp, 15));
 						printout(tempstring);
 					} else {
-						fprintf(stdout, "Generated from the extension identifier of an EUI-48 (MAC): ...:%02x:%02x:%02x\n", (unsigned int) ipv6addr_getoctett(ipv6addrp, 13), (unsigned int) ipv6addr_getoctett(ipv6addrp, 14), (unsigned int) ipv6addr_getoctett(ipv6addrp, 15));
+						fprintf(stdout, "Generated from the extension identifier of an EUI-48 (MAC): ...:%02x:%02x:%02x\n", (unsigned int) ipv6addr_getoctet(ipv6addrp, 13), (unsigned int) ipv6addr_getoctet(ipv6addrp, 14), (unsigned int) ipv6addr_getoctet(ipv6addrp, 15));
 					};
 				} else if ( (typeinfo & IPV6_NEW_ADDR_ISATAP) != 0 )  {
 					for (i = 0; i <= 3; i++) {
-						ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) (i + 12)));
+						ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
 					};
 
 					if ( machinereadable != 0 ) {
@@ -864,7 +899,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 						fprintf(stdout, "Address type contains IPv4 address:\n");
 					};
 					for (i = 0; i <= 3; i++) {
-						ipv4addr_setoctett(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctett(ipv6addrp, (unsigned int) (i + 12)));
+						ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
 					};
 					if ( machinereadable != 0 ) {
 						// printout("IPV4_SOURCE=LINK-LOCAL-IID");
@@ -914,13 +949,8 @@ END:
 
 #ifdef SUPPORT_GEOIP
 		/* GeoIP information */
-		if (use_geoip == 1) {
-			/* C-API at least version 1.3.17 currently doesn't support IPv6, in addition no related database is available */
-			if ((formatoptions & FORMATOPTION_quiet) == 0) {
-				fprintf(stderr, "%s: GeoIP C-API currently misses support of IPv6\n", DEBUG_function_name);
-			};
-
-			// print_geoip(ipv6addrstring, formatoptions, "");
+		if (use_geoip_ipv6 == 1) {
+			print_geoip(ipv6addrstring, formatoptions, "", 6);
 		};
 #endif
 	};
